
-- saved data
MistWeaverData = {};

MistWeaverData.ACTIVE = 1;

MistWeaverData.UNITWIDTH = 100;
MistWeaverData.UNITHEIGHT = 50;
MistWeaverData.COLORTYPE = 1;
MistWeaverData.SORTTYPE = 1;
MistWeaverData.ORIENTATION = 1;
MistWeaverData.OUT_OF_RANGE = "You are out of range!";
MistWeaverData.HIGHLIGHT_RED = 0;
MistWeaverData.HIGHLIGHT_GREEN = 1;
MistWeaverData.HIGHLIGHT_BLUE = 0;
MistWeaverData.SHOW_IN_ALL_STANCES = 0;
MistWeaverData.SHOW_SOLO = 0;

local initialized = false;

local COLOR_TYPE = {
    MW_COLOR_TYPE_HEALTH,
    MW_COLOR_TYPE_CLASS,
}
local SORT_TYPE = {
    MW_SORT_TYPE_ID,
    MW_SORT_TYPE_GROUP,
    MW_SORT_TYPE_NAME,
}
local ORIENTATION = {
    MW_UNIT_SELECT_VERTICAL,
    MW_UNIT_SELECT_HORIZONTAL,
}

-- Controls des Config UI
local controls = {};

function MwConfig_OnLoad(self)
    DEFAULT_CHAT_FRAME:AddMessage("|c333399ffNirriti's|r MistWeaver Config loaded.", 1.0, 1.0, 1.0, 1, 10);
    
    MwConfig_CreateUI(self);    

    self:RegisterEvent("ADDON_LOADED");
    self:RegisterEvent("CVAR_UPDATE");
end

function MwConfig_OnEvent(self, event, ...)
    if (event == "CVAR_UPDATE") then
        local name, value = ...;
        --DEFAULT_CHAT_FRAME:AddMessage("CVAR_UPDATE   "..name.." = "..value, 1.0, 1.0, 1.0, 1, 10);
    end
    if (event == "ADDON_LOADED") then   
        local addonName = ...;
        if (addonName == "MistWeaver") then
            MistWeaver_StartDelay(2, MwConfig_LoadDefaults);
        end
    end
end

function MwConfig_LoadDefaults()
    MistWeaverData.ACTIVE = MistWeaverData.ACTIVE or 1;
    MistWeaverData.UNITWIDTH = MistWeaverData.UNITWIDTH or 100;
    MistWeaverData.UNITHEIGHT = MistWeaverData.UNITHEIGHT or 50;
    MistWeaverData.COLORTYPE = MistWeaverData.COLORTYPE or 1;
    MistWeaverData.SORTTYPE = MistWeaverData.SORTTYPE or 1;
    MistWeaverData.ORIENTATION = MistWeaverData.ORIENTATION or 1;
    MistWeaverData.OUT_OF_RANGE = MistWeaverData.OUT_OF_RANGE or "You are out of range!";
    MistWeaverData.HIGHLIGHT_RED = MistWeaverData.HIGHLIGHT_RED or 0;
    MistWeaverData.HIGHLIGHT_GREEN = MistWeaverData.HIGHLIGHT_GREEN or 1;
    MistWeaverData.HIGHLIGHT_BLUE = MistWeaverData.HIGHLIGHT_BLUE or 0;
    MistWeaverData.SHOW_IN_ALL_STANCES = MistWeaverData.SHOW_IN_ALL_STANCES or 0;
    MistWeaverData.SHOW_SOLO = MistWeaverData.SHOW_SOLO or 0;
    
    initialized = true;
end

function MwConfig_RefreshUI()
    MwConfig_ToggleActiveCheckButton:SetChecked(MistWeaverData.ACTIVE);
    
    MwConfig_UnitFrameWidthSlider:SetValue(MistWeaverData.UNITWIDTH);
    _G[MwConfig_UnitFrameWidthSlider:GetName().."Value"]:SetText(format("%.0f", MistWeaverData.UNITWIDTH));
    
    MwConfig_UnitFrameHeightSlider:SetValue(MistWeaverData.UNITHEIGHT);
    _G[MwConfig_UnitFrameHeightSlider:GetName().."Value"]:SetText(format("%.0f", MistWeaverData.UNITHEIGHT));
    
    UIDropDownMenu_SetSelectedValue(MwConfig_ColorTypeDropDown, MistWeaverData.COLORTYPE);
    UIDropDownMenu_SetText(MwConfig_ColorTypeDropDown, COLOR_TYPE[MistWeaverData.COLORTYPE]);
    
    UIDropDownMenu_SetSelectedValue(MwConfig_SortTypeDropDown, MistWeaverData.SORTTYPE);
    UIDropDownMenu_SetText(MwConfig_SortTypeDropDown, SORT_TYPE[MistWeaverData.SORTTYPE]);
    
    UIDropDownMenu_SetSelectedValue(MwConfig_OrientationDropDown, MistWeaverData.ORIENTATION);
    UIDropDownMenu_SetText(MwConfig_OrientationDropDown, ORIENTATION[MistWeaverData.ORIENTATION]);
    
    MwConfig_WhisperText:SetText(MistWeaverData.OUT_OF_RANGE);
    MwConfig_WhisperText:SetCursorPosition(0);
    
    MwConfig_ToggleShowInAllStancesCheckButton:SetChecked(MistWeaverData.SHOW_IN_ALL_STANCES);
    MwConfig_ToggleShowSoloCheckButton:SetChecked(MistWeaverData.SHOW_SOLO);
    
    MwConfigHighlightColor:SetBackdropColor(MistWeaverData.HIGHLIGHT_RED, MistWeaverData.HIGHLIGHT_GREEN, MistWeaverData.HIGHLIGHT_BLUE);
      
end

function MwConfig_CreateUI(self)    

    -- Hauptfenster
    self.name = "MistWeaver";
    self.addonname = "MistWeaver";
    self.refresh = MwConfig_RefreshUI;
    InterfaceOptions_AddCategory(self);

    local author = GetAddOnMetadata(self.addonname, "Author");
    local version = GetAddOnMetadata(self.addonname, "Version");
    

    local title = self.addonname.." v"..version.." ("..author..")";
    
    local headline = self:CreateFontString(self:GetName().."Title", "ARTWORK", "MwConfigTitleTemplate");
    headline:SetText("|c333399ff"..title.."|r");
    headline:ClearAllPoints(); 
    headline:SetPoint("TOPLEFT", self, "TOPLEFT", 16, -16);
    
    local subtext = self:CreateFontString(self:GetName().."SubText", "ARTWORK", "MwConfigSubTextTemplate");
    subtext:SetText(MW_ADDON_INFO);
    subtext:ClearAllPoints(); 
    subtext:SetPoint("TOPLEFT", self, "TOPLEFT", 16, -48);
    subtext:SetPoint("BOTTOMRIGHT", self, "TOPRIGHT", -16, -85);
    
    MwConfig_AddCheckButton(self, "MwConfig_ToggleActiveCheckButton",
                MW_ON, MwConfig_ToggleActive);
                
    MwConfig_AddSlider(self, "MwConfig_UnitFrameWidthSlider",
                MW_UNIT_FRAME_WIDTH, 50, 200, 1, MwConfig_ChangeUnitFrameWidth);
                
    MwConfig_AddSlider(self, "MwConfig_UnitFrameHeightSlider",
                MW_UNIT_FRAME_HEIGHT, 40, 100, 1, MwConfig_ChangeUnitFrameHeight);
                
    MwConfig_AddDropDown(self, "MwConfig_ColorTypeDropDown",
                MW_UNIT_HEALTHBAR_COLOR, MwConfig_ColorTypeDropDown_Initialize);
                
    MwConfig_AddDropDown(self, "MwConfig_SortTypeDropDown",
                MW_UNIT_SORT_TYPE, MwConfig_SortTypeDropDown_Initialize);
                
    MwConfig_AddDropDown(self, "MwConfig_OrientationDropDown",
                MW_UNIT_FRAME_ORIENTATION, MwConfig_OrientationDropDown_Initialize);
                
    MwConfig_AddEditBox(self, "MwConfig_WhisperText",
                MW_OOR_TEXT, MwConfig_ChangeWhisperText, 250);
                
    MwConfig_AddControl(self, MwConfigHighlightColor);
    
    MwConfig_AddCheckButton(self, "MwConfig_ToggleShowInAllStancesCheckButton",
                MW_SHOW_IN_ALL_STANCES, MwConfig_ToggleShowInAllStances);
    
    MwConfig_AddCheckButton(self, "MwConfig_ToggleShowSoloCheckButton",
                MW_SHOW_SOLO, MwConfig_ToggleShowSolo);
                
end

function MwConfig_AddControl(parent, control, xoffset)

    control:SetParent(parent);

    local liste = controls[parent:GetName()];
    if (not liste) then
        liste = {};
        controls[parent:GetName()] = liste;
    end
    
    local size = table.getn(liste);
    
    if (xoffset) then
        control.xoffset = xoffset;
    end
    
    local ref = parent;
    local relative = "TOPLEFT";
    local x = 15 + (xoffset or 0);
    local y = -90;
    
    if (size > 0) then
        ref = liste[size];
        relative = "BOTTOMLEFT";
        x = (xoffset or 0) - (ref.xoffset or 0);
        y = -14;
    end
    
    table.insert(liste, control);
    
    control:ClearAllPoints(); 
    control:SetPoint("TOPLEFT", ref, relative, x, y);
    
    return control;
end

function MwConfig_AddEditBox(parent, name, text, onTextChanged, width, xoffset)
    local editBox = CreateFrame("EditBox", name, parent, "MwConfigEditBoxTemplate");
    editBox.text:SetText(text);
    
    editBox:SetScript("OnTextChanged", onTextChanged);
    
    if (width) then
        editBox:SetWidth(width);
    end
    
    return MwConfig_AddControl(parent, editBox, xoffset);
end

function MwConfig_AddCheckButton(parent, name, text, onClick, size, xoffset)
    local checkButton = CreateFrame("CheckButton", name, parent, "MwConfigCheckButtonTemplate");
    checkButton.text:SetText(text);
    
    checkButton:SetScript("OnClick", onClick);
    
    if (size) then
        checkButton:SetSize(size, size);
    end
    
    return MwConfig_AddControl(parent, checkButton, xoffset);
end

function MwConfig_AddSlider(parent, name, text, min, max, step, onValueChanged, xoffset)
    local slider = CreateFrame("Slider", name, parent, "MwConfigSliderTemplate");
    
    slider:SetScript("OnValueChanged", onValueChanged);
    
    slider:SetMinMaxValues(min, max);
    slider:SetValueStep(step);
    
    local label = _G[slider:GetName().."Text"];
    label:SetText(text);
    label:ClearAllPoints();
    label:SetPoint("LEFT", slider, "RIGHT", 15, 0);
    label:SetFontObject("GameFontNormal");
                        
    local textLow = _G[slider:GetName().."Low"];
    local textHigh = _G[slider:GetName().."High"];
    textLow:SetText(min);
    textHigh:SetText(max);   
                         
    return MwConfig_AddControl(parent, slider, xoffset);
end

function MwConfig_AddDropDown(parent, name, text, onInitialize, width, xoffset)
    local dropdown = CreateFrame("Frame", name, parent, "MwConfigDropDownMenuTemplate");
    
    dropdown.label:SetText(text);
    
    UIDropDownMenu_Initialize(dropdown, onInitialize);
    UIDropDownMenu_JustifyText(dropdown, "LEFT");
    
    if (width) then
        UIDropDownMenu_SetWidth(dropdown, width); 
    end
    
    return MwConfig_AddControl(parent, dropdown, xoffset);
end

function MwConfig_ToggleActive()
    if (MistWeaverData.ACTIVE == 1) then
        MistWeaverData.ACTIVE = 0;
    else
        MistWeaverData.ACTIVE = 1;
    end
    
    MwConfig_RefreshUI();
    MistWeaver_UpdateActiveState();
end

function MwConfig_ToggleShowInAllStances()
    if (MistWeaverData.SHOW_IN_ALL_STANCES == 1) then
        MistWeaverData.SHOW_IN_ALL_STANCES = 0;
    else
        MistWeaverData.SHOW_IN_ALL_STANCES = 1;
    end
    
    MwConfig_RefreshUI();
    MistWeaver_UpdateActiveState();
end

function MwConfig_ToggleShowSolo()
    if (MistWeaverData.SHOW_SOLO == 1) then
        MistWeaverData.SHOW_SOLO = 0;
    else
        MistWeaverData.SHOW_SOLO = 1;
    end
    
    MwConfig_RefreshUI();
    MistWeaver_UpdateActiveState();
end

function MwConfig_ChangeUnitFrameWidth(self)

    if (MistWeaverData.UNITWIDTH ~= self:GetValue()) then
        MistWeaverData.UNITWIDTH = self:GetValue();
    
        MwConfig_RefreshUI();
        MistWeaver_UpdateUnitFrameSize();
    end
end

function MwConfig_ChangeUnitFrameHeight(self)

    if (MistWeaverData.UNITHEIGHT ~= self:GetValue()) then
        MistWeaverData.UNITHEIGHT = self:GetValue();
    
        MwConfig_RefreshUI();
        MistWeaver_UpdateUnitFrameSize();
    end
end

function MwConfig_ColorTypeDropDown_Initialize(self)
    local selectedValue = UIDropDownMenu_GetSelectedValue(self);
    
    for i = 1, #COLOR_TYPE do
        local info = UIDropDownMenu_CreateInfo();
        info.text = COLOR_TYPE[i];
        info.value = i;
        info.func = function()
                MwConfig_ToggleColorType(i);
            end
        UIDropDownMenu_AddButton(info);
    end
end

function MwConfig_ToggleColorType(index)
    MistWeaverData.COLORTYPE = index;
    MwConfig_RefreshUI();
end

function MwConfig_SortTypeDropDown_Initialize(self)
    local selectedValue = UIDropDownMenu_GetSelectedValue(self);
    
    for i = 1, #SORT_TYPE do
        local info = UIDropDownMenu_CreateInfo();
        info.text = SORT_TYPE[i];
        info.value = i;
        info.func = function()
                MwConfig_ToggleSortType(i);
            end
        UIDropDownMenu_AddButton(info);
    end
end

function MwConfig_ToggleSortType(index)
    MistWeaverData.SORTTYPE = index;
    MwConfig_RefreshUI();
    MistWeaver_RebindUnitFrames();
end

function MwConfig_OrientationDropDown_Initialize(self)
    local selectedValue = UIDropDownMenu_GetSelectedValue(self);
    
    for i = 1, #ORIENTATION do
        local info = UIDropDownMenu_CreateInfo();
        info.text = ORIENTATION[i];
        info.value = i;
        info.func = function()
                MwConfig_ToggleOrientation(i);
            end
        UIDropDownMenu_AddButton(info);
    end
end

function MwConfig_ToggleOrientation(index)
    MistWeaverData.ORIENTATION = index;
    MwConfig_RefreshUI();
    MistWeaver_RebindUnitFrames();
    MistWeaver_SetupUnitFramePositions();
end

function MwConfig_ChangeWhisperText(self)
    if (initialized) then
        MistWeaverData.OUT_OF_RANGE = self:GetText();
    end
end

function MwConfig_SelectHighlightColor()
    ColorPickerFrame.previousValues = {r = MistWeaverData.HIGHLIGHT_RED, g = MistWeaverData.HIGHLIGHT_GREEN, b = MistWeaverData.HIGHLIGHT_BLUE};
    ColorPickerFrame.func = MwConfig_SetHighlightColor;
    ColorPickerFrame.cancelFunc = MwConfig_CancelHighlightColor;
    ColorPickerFrame:SetColorRGB(MistWeaverData.HIGHLIGHT_RED, MistWeaverData.HIGHLIGHT_GREEN, MistWeaverData.HIGHLIGHT_BLUE);
    ShowUIPanel(ColorPickerFrame);
end

function MwConfig_SetHighlightColor()
    local r,g,b = ColorPickerFrame:GetColorRGB();
    MistWeaverData.HIGHLIGHT_RED = r;
    MistWeaverData.HIGHLIGHT_GREEN = g;
    MistWeaverData.HIGHLIGHT_BLUE = b;
    
    MwConfig_RefreshUI();
end

function MwConfig_CancelHighlightColor(prevValues)
    MistWeaverData.HIGHLIGHT_RED = prevValues.r;
    MistWeaverData.HIGHLIGHT_GREEN = prevValues.g;
    MistWeaverData.HIGHLIGHT_BLUE = prevValues.b; 
    
    MwConfig_RefreshUI();
end


